require ("config")
require ("data_tables")

function TableIsEmpty( table )

	local num = 0

	for k, v in pairs ( table ) do

		num = num + 1

	end

	return num == 0

end

function RequireFiles ( files )

	for k, v in pairs ( files ) do
	
		require ( v )
	
	end

end

function ItemIsVanilla ( item )

	return Prototypes[item] == nil

end

function ItemEnabled ( name, count )
		
		local item = name
		local item_name = string.gsub( name, "-", "_" )
		local number = count
		
		if ItemIsVanilla ( item_name ) then
		
			return { item, number }
		
		end
		
		while Prototypes[item_name]["enabled"] == false do
			
			number = ( number * Prototypes[item_name]["replace"].count )
			item = Prototypes[item_name]["replace"].name
			item_name = Prototypes[item_name]["replace"].name
			
			
			if ItemIsVanilla ( item_name ) then
			
				return { item, number }
			
			end
			
		end
		
		return { Prototypes[item_name]["names"].item, number }

end

function FluidEnabled ( name, count )
		
		local item = name
		local item_name = string.gsub( name, "-", "_" )
		
		if ItemIsVanilla ( item_name ) then
		
			return item
		
		end
		
		while Prototypes[item_name]["enabled"] == false do
		
			item = Prototypes[item_name]["replace"].name
			item_name = Prototypes[item_name]["replace"].name
			
			if ItemIsVanilla ( item_name ) then
			
				return item
			
			end
			
		end
		
		return Prototypes[item_name]["names"].item

end

function CheckRecipe ( items )

	local recipe = {}
	local new_recipe = {}

	for k, v in pairs ( items ) do
		
		if v.fluid == true then
		
			table.insert( recipe, k, { type = "fluid", name = FluidEnabled( v.name ), amount = v.count } )
			
			
		else
		
			local item_enabled = ItemEnabled( v.name, v.count )
		
			table.insert( recipe, k, { type = "item", name = item_enabled[1], amount = item_enabled[2] } )
		
		end
	
	end
	
	local function recipe_has_value( element, count )
	
		for i, j in pairs(new_recipe) do
		
			if j.name == element then
			
				new_recipe[i].amount = new_recipe[i].amount + count
			
				return true
				
			end
			
		end
		
		return false
	
	end
	
	
	for k, v in pairs ( recipe ) do
	
		if recipe_has_value( v.name, v.amount ) ~= true then
		
			table.insert( new_recipe, { type = v.type, name = v.name, amount = v.amount } )
		
		end
	
	end
	
	return new_recipe

end

function CheckResearchRecipe ( items )

	local researchrecipe = {}

	for k, v in pairs ( items ) do

		if Prototypes[v.name] == nil then 
		
			table.insert( researchrecipe, k, { v.name, v.count } )
		
		else
	
			if Prototypes[v.name]["enabled"] == true then
			
				table.insert( researchrecipe, k, { v.name, v.count } )
			
			end
		
		end
	
	end
	
	if TableIsEmpty( researchrecipe ) then
	
		researchrecipe = { {"science-pack-1",1} }
		
	end
	
	return researchrecipe

end

function CheckResearchUnlock( research )

	local recipes = {}

	for k, v in pairs ( research ) do
	
		if Research[v].type == "unlock-recipe" then
		
			for _, rec in pairs( Research[v].recipes ) do
		
				if Prototypes[rec] == nil then
				
					table.insert( recipes, { type = "unlock-recipe", recipe = rec } )
				
				else
		
					if Prototypes[rec]["enabled"] == true then
				
						table.insert( recipes, { type = "unlock-recipe", recipe = Prototypes[rec]["names"].recipe } )

						for i = 1, 20 do
						
							if Prototypes[rec]["names"]["recipe_"..i] then
							
								table.insert( recipes, { type = "unlock-recipe", recipe = Prototypes[rec]["names"]["recipe_"..i] } )
							
							end
						
						end
				
					end
				
				end
			
			end
		
		end
	
	end
	
	return recipes

end

function CheckVanillaResearchUnlock( research )

	local recipes = {}

	for k, v in pairs ( research ) do
	
		if Modifications[v].type == "unlock-recipe" then
		
			for _, rec in pairs( Modifications[v].recipes ) do
		
				if Prototypes[rec] == nil then
				
					table.insert( recipes, { type = "unlock-recipe", recipe = rec } )
				
				else
		
					if Prototypes[rec]["enabled"] == true then
				
						table.insert( recipes, { type = "unlock-recipe", recipe = Prototypes[rec]["names"].recipe } )

						for i = 1, 20 do
						
							if Prototypes[rec]["names"]["recipe_"..i] then
							
								table.insert( recipes, { type = "unlock-recipe", recipe = Prototypes[rec]["names"]["recipe_"..i] } )
							
							end
						
						end
				
					end
				
				end
			
			end
		
		end
	
	end
	
	return recipes

end

function ResearchIsVanilla ( res )

	return Research[res] == nil

end

function Prerequisites ( requisite )
		
		local req = requisite
		
		if ResearchIsVanilla ( req ) then
		
			return req
		
		end
		
		while Research[req]["enabled"] == false do

			req = Research[req]["replace"]
			
			if ResearchIsVanilla ( req ) then
			
				return req
			
			end

		end
		
		return Research[req]["names"].technology

end

function CheckResearchRegistration( prerequisite, requisites )

	for k, v in pairs( requisites ) do
	
		if prerequisite == v then
		
			return false
			
		end
	
	end
	
	return true

end

function CheckResearchPrerequisites( prerequisites )

	local requisites = {}

	for k, v in pairs ( prerequisites ) do
				
		if CheckResearchRegistration( Prerequisites( v ), requisites ) then
				
			table.insert( requisites, k, Prerequisites( v ) )
		
		end
		
	end
	
	return requisites

end

function CheckResultsMining( mines, replacement )

	local function ReturnMines( item, mintyp, minprob )
	
	return {
	type = mintyp,
    name = item,
    amount = 1,
    probability = minprob,
	}
	
	end

	local minerecipe = {}

	for k, v in pairs ( mines ) do

		if Prototypes[v.name] == nil then 
		
			if v.mineprobability > 0 then
		
				table.insert( minerecipe, k, ReturnMines( v.name, "item", v.mineprobability ) )
			
			end
		
		else
	
			if Prototypes[v.name]["enabled"] == true then
			
				if v.mineprobability > 0 then
			
					table.insert( minerecipe, k, ReturnMines( Prototypes[v.name]["names"].item, v.type, v.mineprobability ) )
				
				end
			
			end
		
		end
	
	end
	
	if TableIsEmpty( minerecipe ) then
	
		minerecipe = { ReturnMines( replacement.name, replacement.type, replacement.mineprobability ) }
		
	end
	
	return minerecipe



end

