
local PipeButtons = {
	
	CloseButton = 
	{
		name = "CloseButton", 
		excluded = true,
		output = function ( event )
		
			local ply = game.players[event.player_index]
			RemovePipeGui( ply, "center", "pipe-junctions" )
		
		end,
	},	
	Straight = 
	{
		name = "Straight", 
		sprite = "pipe_straight",
		frame_table = "Pipes",
		flow = "StraightPipe",
		label = { name = "StraightLabel", caption = "Straight pipe" },
		output = function ( event )
		
			local ply = game.players[event.player_index]
			BuildPipe( ply, event.player_index, "-straight" )
		
		end,
	},	
	Elbow = 
	{
		name = "Elbow", 
		sprite = "pipe_elbow",
		frame_table = "Pipes",
		flow = "ElbowPipe",
		label = { name = "ElbowLabel", caption = "Elbow pipe" },
		output = function ( event )
		
			local ply = game.players[event.player_index]
			BuildPipe( ply, event.player_index, "-elbow" )
		
		end,
	},	
	Junction = 
	{
		name = "Junction", 
		sprite = "pipe_junction",
		frame_table = "Pipes",
		flow = "JunctionPipe",
		label = { name = "JunctionLabel", caption = "Junction pipe" },
		output = function ( event )
		
			local ply = game.players[event.player_index]
			BuildPipe( ply, event.player_index, "-junction" )
		
		end,
	},	
	Check = 
	{
		name = "Check", 
		sprite = "pipe_check",
		frame_table = "Pipes",
		flow = "CheckPipe",
		label = { name = "CheckLabel", caption = "Circuit pipe" },
		output = function ( event )
		
			local ply = game.players[event.player_index]
			BuildPipe( ply, event.player_index, "-check" )
		
		end,
	},	
	Overflow = 
	{
		name = "Overflow", 
		sprite = "pipe_overflow",
		frame_table = "Pipes",
		flow = "OverflowPipe",
		label = { name = "OverflowLabel", caption = "Overflow pipe" },
		output = function ( event )
		
			local ply = game.players[event.player_index]
			BuildPipe( ply, event.player_index, "-overflow" )
		
		end,
	},
}

function BuildPipe( ply, id, type )

	local pipe = Pipes[id]
	local base = pipe.name
	local base = string.gsub( base, "-straight", "" )
	local base = string.gsub( base, "-elbow", "" )
	local base = string.gsub( base, "-junction", "" )
	local base = string.gsub( base, "-check", "" )
	local base = string.gsub( base, "-overflow", "" )
	
	local X = pipe.position.x
	local Y = pipe.position.y
	local straightpipe = pipe.surface.create_entity{ name = base..""..type, position = { X, Y }, direction = pipe.direction, force = game.forces.player }
	straightpipe.fluidbox[1] = pipe.fluidbox[1]
	
	pipe.destroy()
	Pipes[id] = straightpipe


end

local function GuiButtonClickPipe( event )

	if game.players[event.player_index].gui.center["pipe-junctions"] == nil then return end

	if Pipes[event.player_index] == nil or 
	Pipes[event.player_index].valid == false or 
	event.element.valid ~= true
	then 
	
		DebugPrint( "Triggered pipe" )
		RemovePipeGui( game.players[event.player_index], "center", "pipe-junctions" )
		return
		
	end

	if event.element.type == "button" or event.element.type == "sprite-button" then
	
		if PipeButtons[event.element.name] then
		
			if PipeButtons[event.element.name].output then
		
				PipeButtons[event.element.name].output( event )
			
			end
		
		end
	
		return
	
	end

	--[[if event.element.type == "checkbox" then

		if Checkboxes[event.element.name].output then
	
			Checkboxes[event.element.name].output( event )
		
		end
		
		return
	
	end--]]


end

function RemovePipeGui( ply, pos, frame_name )
		
	if ply.gui[pos][frame_name] and Pipes[ply.index] then Pipes[ply.index] = nil end

	for k, v in pairs( { frame_name, "inserter-adjustment" } ) do
	
		if ply.gui[pos][v] then ply.gui[pos][v].destroy() end
	
	end

end

function InitPipeGui( ply, ent )

	RemovePipeGui( ply, "center", "pipe-junctions" )

	frame = ply.gui["center"].add{type="frame", name="pipe-junctions", caption = "Pipe", direction='horizontal'} --style='naked_frame_style'

	frame.add( { type="frame", name="Pipes", direction="vertical",      caption="Pipe junctions"} )
	frame.add( { type="button", name=PipeButtons["CloseButton"].name, caption="X", } ) --style = "button_style"
	frame.Pipes.add( {type="flow",name="StraightPipe",caption="",direction="horizontal"} )
	frame.Pipes.add( {type="flow",name="ElbowPipe",caption="",direction="horizontal"} )
	frame.Pipes.add( {type="flow",name="JunctionPipe",caption="",direction="horizontal"} )
	frame.Pipes.add( {type="flow",name="CheckPipe",caption="",direction="horizontal"} )
	frame.Pipes.add( {type="flow",name="OverflowPipe",caption="",direction="horizontal"} )
	
	for k, v in pairs( PipeButtons ) do
		
		if v.excluded ~= true then
		
			--if v.output ~= nil then
			
				frame[v.frame_table][v.flow].add( { type="sprite-button", name=v.name, caption=v.caption or "", style = "pipe_button", sprite=v.sprite } )
				frame[v.frame_table][v.flow].add( { type="label", name=v.label.name, caption=v.label.caption or "", } ) --style = "label_style"
			
			--else
			
				--frame[v.frame_table][v.flow].add( { type="button", name=v.name, caption=v.caption or "", state = false, style = "inserter_direction_middle" } )
			
			--end
		
		end
		
	end
	

end

function AdjustPipe( ply, ent )

	InitPipeGui( ply, ent )

end

Pipes = {}



function AdjustPipeProperties ( event )

	local player = game.players[event.player_index]
	local id = event.player_index
	local ent = player.selected
	local basename = function ( name )
	
		local base = name
		base = string.gsub( base, "-straight", "" )
		base = string.gsub( base, "-elbow", "" )
		base = string.gsub( base, "-junction", "" )
		base = string.gsub( base, "-check", "" )
		base = string.gsub( base, "-overflow", "" )
		name = base
	
		return name
	
	end

	if ( ent ~= nil ) and 
	( ( ent.type == "pipe" and game.entity_prototypes[basename(ent.name).."-straight"] ~= nil ) or ( ent.type == "pipe-to-ground" and game.entity_prototypes[basename(ent.name).."-straight"] ~= nil ) or ( ent.type == "storage-tank" and game.entity_prototypes[basename(ent.name).."-straight"] ~= nil ) ) and 
	( player.can_reach_entity(ent) ) then

		RemovePipeGui( player, "center", "pipe-junctions" )
	
		table.insert( Pipes, id, ent )
		AdjustPipe( player, ent )

	end


end

AddEventFunction( "adjust_object_properties", AdjustPipeProperties )
AddEventFunction( "on_gui_click", GuiButtonClickPipe )

